/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>,
 * Benoit Valot <Benoit.Valot@moulon.inra.fr>
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline;

import java.util.Collection;
import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.Protein;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMass;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationXtandemFile;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetPhospho;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;

public class Utils {
	private static final Logger logger = Logger.getLogger(Utils.class);

	public final static double phospho_mass = (float) 79.96633;
	/*
	 * m(1H) = 1.00727646677 u = mass of proton; charge +1 m(1H+e-) = 1.00782504
	 * u = mass of proton + mass of electron
	 */
	public final static double mhplus = (double) 1.00727646677;

	/*
	 * 
	 * Monoisotopic mass from : http://www.unimod.org/masses.html 1.007825035
	 */
	public final static double mhelectron = (double) 1.007825035;

	public final static double mDeltaC12C13 = (double) 1.0033548378;

	public final static double mneutron = (double) 1.0086655;

	public final static double onemillion = (double) 1000000;

	static private String getPappsoGroupingId(Integer num) {
		int size = (int) java.lang.Math.log10(num);
		size += 97;
		String code = "" + (char) size;
		code += "" + num;
		return (code);
	}

	static public String getPappsoGroupingNumber(Group group, SubGroup sg, int k)
			throws MSMSException {
		if (group == null) {
			String message = "group is null";
			logger.error(message);
			throw new MSMSException(message);
		}
		if (sg == null) {
			String message = "sg is null";
			logger.error(message);
			throw new MSMSException(message);
		}

		return getPappsoGroupingId(group.getNumber()) + "."
				+ getPappsoGroupingId(sg.getNumber()) + "."
				+ getPappsoGroupingId(k);
	}

	static public String getPappsoPhosphoPeptideMassId(Group group,
			PeptideMass peptideMass) throws MSMSException {
		if (peptideMass == null) {
			String message = "peptideMass is null";
			logger.error(message);
			throw new MSMSException(message);
		}
		if (group == null) {
			String message = "group is null";
			logger.error(message);
			throw new MSMSException(message);
		}

		SubGroup sg = peptideMass.getSubGroupSet().iterator().next();
		if (sg == null) {
			String message = "sg is null";
			logger.error(message);
			throw new MSMSException(message);
		}
		return "pep" + getPappsoGroupingId(group.getNumber())
				+ getPappsoGroupingId(sg.getNumber())
				+ getPappsoGroupingId(peptideMass.getNumber());
	}

	static public String getPappsoPeptideMassId(Group group,
			PeptideMass peptideMass) throws MSMSException {
		if (peptideMass == null) {
			String message = "peptideMass is null";
			logger.error(message);
			throw new MSMSException(message);
		}
		if (group == null) {
			String message = "group is null";
			logger.error(message);
			throw new MSMSException(message);
		}
		return "pep" + getPappsoGroupingId(group.getNumber())
				+ getPappsoGroupingId(peptideMass.getNumber());
	}

	static public String getPappsoGroupId(Group group) throws MSMSException {
		if (group == null) {
			String message = "group is null";
			logger.error(message);
			throw new MSMSException(message);
		}
		return getPappsoGroupingId(group.getNumber());
	}

	public static String getPappsoSubGroupId(Group group, SubGroup sg)
			throws MSMSException {
		if (group == null) {
			String message = "group is null";
			logger.error(message);
			throw new MSMSException(message);
		}
		if (sg == null) {
			String message = "sg is null";
			logger.error(message);
			throw new MSMSException(message);
		}

		return getPappsoGroupingId(group.getNumber()) + "."
				+ getPappsoGroupingId(sg.getNumber());
	}

	public static String collectionToString(Collection<?> obsPhosphoPos) {
		String toWrite = "";
		for (Object item : obsPhosphoPos) {
			toWrite += " " + item;
		}
		return toWrite.trim();
	}

	public static String getPappsoPhosphoIslandId(Group group, SubGroup sg,
			HashSampleScanSetPhospho phospho) throws MSMSException {
		Protein prot = (Protein) phospho.getGrpProtMatch().getGrpProtein();
		return getPappsoGroupingNumber(group, sg, prot.getPhosphoId());
	}

	public static HashSet<IdentificationXtandemFile> getXtandemFileList(
			Identification identification) {
		// if we can retrieve original informations in xml xtandem results
		boolean originalIdentificationResultFiles = true;
		HashSet<IdentificationXtandemFile> xtSourceFiles = new HashSet<IdentificationXtandemFile>();

		for (MsRun msRun : identification.getMsRunSet().getMsRunList()) {
			for (IdentificationDataSource sourceFile : msRun
					.getIdentificationDataSourceList()) {
				IdentificationXtandemFile xtFile = null;
				try {
					xtFile = (IdentificationXtandemFile) sourceFile;
				} catch (Exception e) {
					String message = "cast failed : " + e.getMessage();
					logger.warn(message);
					originalIdentificationResultFiles = false;
					continue;
				}
				if (!xtFile.exists()) {
					String message = "missing X!tandem result file : "
							+ xtFile.getAbsolutePath();
					logger.warn(message);
					originalIdentificationResultFiles = false;
				}

				if (originalIdentificationResultFiles == false) {
					return (null);
				}
				if (!xtSourceFiles.contains(xtFile)) {
					logger.debug("add file : " + xtFile.getAbsolutePath());
					xtSourceFiles.add(xtFile);
				}
			}
		}
		return (xtSourceFiles);
	}

	public static float getAaMass(Character aa) {

		float mass = 0;
		switch (aa) {
		case 'W':
			mass = (float) 186.0793;
			break;
		case 'G':
			mass = (float) 57.0214;
			break;
		case 'A':
			mass = (float) 71.0371;
			break;
		case 'S':
			mass = (float) 87.0320;
			break;
		case 'P':
			mass = (float) 97.0527;
			break;
		case 'V':
			mass = (float) 99.0684;
			break;
		case 'T':
			mass = (float) 101.0476;
			break;
		case 'L':
			mass = (float) 113.0840;
			break;
		case 'I':
			mass = (float) 113.0840;
			break;
		case 'N':
			mass = (float) 114.0429;
			break;
		case 'D':
			mass = (float) 115.0269;
			break;
		case 'K':
			mass = (float) 128.0949;
			break;
		case 'Q':
			mass = (float) 128.0585;
			break;
		case 'E':
			mass = (float) 129.0425;
			break;
		case 'M':
			mass = (float) 131.0404;
			break;
		case 'H':
			mass = (float) 137.0589;
			break;
		case 'F':
			mass = (float) 147.0684;
			break;
		case 'R':
			mass = (float) 156.1011;
			break;
		case 'C':
			mass = (float) 103.00919;
			// addModification((float) 57.021464);
			break;
		case 'Y':
			mass = (float) 163.0633;
			break;
		default:
		}
		return mass;
	}
}
