package fr.inra.pappso.xtandempipeline.grouping;

import java.util.ArrayList;
import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpMsSample;

public class Group extends HashKey4GroupsSet implements Comparable<Group> {

	private static final long serialVersionUID = -917565136462863548L;

	private static final Logger logger = Logger.getLogger(Group.class);

	private SubGroupSet subGroupList = new SubGroupSet();

	private Integer number = null;

	public Group(SubGroup subGroup) throws Exception {
		subGroupList.add(subGroup);
		HashKey4GroupsSet peptideSet = subGroup.getHashKey4GroupsSet();
		this.addAll(peptideSet);
	}

	@Override
	public String toString() {
		String groupStr = new String();
		for (SubGroup sg : subGroupList) {
			groupStr = groupStr + sg.toString();
		}
		return groupStr;
	}

	@Override
	public boolean equals(Object InCompare) {
		if (this == InCompare)
			return (true);
		return false;
	}

	public boolean fusionWith(Group incomingGroup) throws Exception {
		// fusion with the incoming group if this contains common elements
		// AND if removing non informative elements does not impact grouping
		boolean fusion = false;

		if (this.containsCommonHashKey4GroupsWith(incomingGroup)) {
			fusion = true;
		}

		if (fusion) {
			this.addAll(incomingGroup);
			subGroupList.addAll(incomingGroup.subGroupList);
		}
		return fusion;
	}

	private boolean containsCommonHashKey4GroupsWith(Group incomingGroup)
			throws Exception {
		for (HashKey4Groups hashPeptide : incomingGroup) {
			if (hashPeptide == null) {
				String message = "hashPeptide is null";
				logger.error(message);
				throw new Exception(message);
			}
			if (this.contains(hashPeptide)) {
				logger.debug("launch fusion : this peptide was found in group "
						+ hashPeptide.toString());
				return true;
			} else {
				// logger.debug(" hashPeptide " + hashPeptide +
				// "("+hashPeptide.hashCode()+") is NOT in group ");
			}
		}
		return false;
	}

	public int compareTo(Group o) {
		int i = o.getHashSampleScanSet().size()
				- this.getHashSampleScanSet().size();
		if (i != 0)
			return i;
		return (this.getHashSampleScanSet().compareTo(o.getHashSampleScanSet()));
	}

	public HashSampleScanSet getHashSampleScanSet() {
		HashSampleScanSet sampleScanSet = new HashSampleScanSet();
		for (SubGroup sg : this.subGroupList) {
			sampleScanSet.addAll(sg);
		}
		return sampleScanSet;
	}

	protected void setNumber(int i) {
		number = i;
	}

	public Integer getNumber() {
		return number;
	}

	public void loggerDebug() {
		logger.debug("group num : " + this.getNumber());
		for (SubGroup sg : this.subGroupList.getSubGroupList()) {
			logger.debug("sub group " + sg.getNumber());

			sg.loggerDebug();
		}
	}

	protected boolean removeNonInformative() throws Exception {
		ArrayList<SubGroup> sgList = this.subGroupList.getSubGroupList();
		for (int i = sgList.size() - 1; i >= 0; i--) {
			HashSampleScanSetProt hashProt = sgList.get(i)
					.getHashSampleScanSetProtList().get(0);
			int count = this.getNumberOfSpecificSpectraInSubGroup(hashProt);
			if (count == 0) {
				this.subGroupList.remove(sgList.get(i));
				logger.debug("Remove non informative subgroup "
						+ sgList.get(i).getHashSampleScanSetProtList().get(0)
								.getGrpProtMatch().getGrpProtein()
								.getAccession());
				return (true);
			}
		}
		return (false);
	}

	public SubGroupSet getSubGroupSet() {
		return this.subGroupList;
	}

	public int getNumberOfSpecificSpectraInSubGroup(
			HashSampleScanSetProt hashProt) {
		int count = 0;
		for (HashSampleScan hash : hashProt) {
			int view = 0;
			for (SubGroup sg : this.subGroupList) {
				if (sg.contains(hash)) {
					view++;
				}
			}
			if (view == 1) {
				count++;
			}
		}
		return (count);
	}

	public int getNumberOfSpecificUniquePeptideInSubGroup(
			HashSampleScanSetProt hashProt) throws MSMSException {
		HashSet<String> sequences = new HashSet<String>();
		for (HashSampleScan hash : hashProt) {
			int view = 0;
			for (SubGroup sg : this.subGroupList) {
				if (sg.contains(hash)) {
					view++;
				}
			}
			if (view == 1) {
				sequences.add(hashProt.getGrpProtMatch()
						.getBestPeptideToHashSampleScan(hash).getSequence());
			}
		}
		return (sequences.size());
	}

	/**
	 * count the number of specific spectra for a protein in one sample
	 * 
	 * @param hashProt
	 * @param sample
	 * @return
	 */
	public int getNumberOfSpecificPeptideInSubGroupToSample(
			HashSampleScanSetProt hashProt, GrpMsSample sample) {
		int count = 0;
		for (HashSampleScan hash : hashProt) {
			if (hash.getGrpMsSample().isIdenticalTo(sample)) {
				int view = 0;
				for (SubGroup sg : this.subGroupList) {
					if (sg.contains(hash)) {
						view++;
					}
				}
				if (view == 1) {
					count++;
				}
			}
		}
		return (count);
	}

	public SubGroupSet getSubGroupContaining(HashSampleScan hash) {
		SubGroupSet contain = new SubGroupSet();
		for (SubGroup sg : this.subGroupList) {
			if (sg.contains(hash)) {
				contain.add(sg);
			}
		}
		return (contain);
	}

	public HashSet<Group> reGroup() throws Exception {
		GroupSet newGroupSet = new GroupSet();
		for (SubGroup sg : this.subGroupList) {
			newGroupSet.addSubGroup(sg);
		}
		logger.debug("new size after reGroup : " + newGroupSet.size());

		return newGroupSet.getHashSetOfGroups();
	}

	public int getNumberOfSpectraInGrpMsSample(GrpMsSample msSample) {
		int count = 0;
		for (HashSampleScan hash : this.getHashSampleScanSet()) {
			if (hash.getGrpMsSample().isIdenticalTo(msSample)) {
				count++;
			}
		}
		return (count);
	}

	/**
	 * get the number of specific unique peptide sequence for a subgroup in a
	 * group by MS sample
	 * 
	 * @param hashProt
	 * @param sample
	 * @return
	 * @throws MSMSException
	 */
	public int getNumberOfSpecificUniquePeptideInSubGroupGrpMsSample(
			HashSampleScanSetProt hashProt, GrpMsSample sample)
			throws MSMSException {
		HashSet<String> sequences = new HashSet<String>();
		for (HashSampleScan hash : hashProt) {
			if (hash.getGrpMsSample().isIdenticalTo(sample)) {
				int view = 0;
				for (SubGroup sg : this.subGroupList) {
					if (sg.contains(hash)) {
						view++;
					}
				}
				if (view == 1) {
					sequences
							.add(hashProt.getGrpProtMatch()
									.getBestPeptideToHashSampleScan(hash)
									.getSequence());
				}
			}
		}
		return (sequences.size());
	}

}
