/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Benoit Valot <Benoit.Valot@moulon.inra.fr>
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.output.SpreadSheets;

import java.io.IOException;
import java.io.OutputStream;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.stream.XMLStreamException;

import org.apache.log4j.Logger;

import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.libodsstream.OdsDocWriterInterface;
import fr.inra.pappso.xtandempipeline.Utils;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.Protein;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;

public class OdsProteinOutput {
	private static final Logger logger = Logger
			.getLogger(OdsProteinOutput.class);

	protected OdsDocWriterInterface odsTable;

	protected OdsProteinOutput() {

	}

	public OdsProteinOutput(OutputStream odsDoc) throws MSMSException {
		try {
			odsTable = new OdsDocWriter(odsDoc);
			odsTable.writeSheet("proteins");
		} catch (IOException e) {
			String message = "IOException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (DatatypeConfigurationException e) {
			String message = "DatatypeConfigurationException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public OdsProteinOutput(OdsDocWriterInterface odsTable)
			throws MSMSException {
		if (odsTable == null) {
			String message = "odsTable is null";
			logger.error(message);
			throw new MSMSException(message);
		}
		try {
			this.odsTable = odsTable;
			odsTable.writeSheet("proteins");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void close() throws MSMSException {
		try {
			odsTable.close();
		} catch (Exception e) {
			String message = "Exception closing OdsProteinOutput "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeHeaders(Identification ident) throws MSMSException {
		try {
			// Group ID Sub-group ID Protein ID Description log(E value)
			// Coverage MW Spectra Specific uniques Uniques PAI Redundancy

			odsTable.writeLine();
			odsTable.writeCell("Group ID");
			odsTable.writeCell("Sub-group ID");
			odsTable.writeCell("Protein ID");
			odsTable.writeCell("Description");
			odsTable.writeCell("log(E value)");
			odsTable.writeCell("Coverage");
			odsTable.writeCell("MW");
			odsTable.writeCell("Spectra");
			odsTable.writeCell("Specific");
			odsTable.writeCell("Uniques");
			odsTable.writeCell("Specific uniques");
			odsTable.writeCell("Theoretical number of tryptic peptides");
			odsTable.writeCell("PAI");
			odsTable.writeCell("emPAI");
			odsTable.writeCell("Sub-group proteins");
			odsTable.writeCell("number of MS samples");

		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeHeaders() "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeHeaders() " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}

	}

	public void write(Identification ident) throws MSMSException {
		try {

			this.writeHeaders(ident);
			for (Group group : ident.getGrouping().getGroupList()) {
				for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {

					// proteins
					int k = 1;
					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtList()) {
						this.writeOneProtein(hashProt, group, sg, k);
						k++;
					}
				}

			}
			odsTable.writeLine();
			odsTable.writeLine();

		} catch (XMLStreamException e) {
			String message = "XMLStreamException in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeOneProtein(HashSampleScanSetProt hashProt, Group group,
			SubGroup sg, int k) throws MSMSException {
		try {
			logger.debug("writeOneProtein begin "
					+ Utils.getPappsoGroupId(group)
					+ Utils.getPappsoSubGroupId(group, sg) + k);
			odsTable.writeLine();

			Match match = (Match) hashProt.getGrpProtMatch();

			Protein prot = match.getGrpProtein();
			odsTable.writeCell(Utils.getPappsoGroupId(group));
			odsTable.writeCell(Utils.getPappsoSubGroupId(group, sg));
			odsTable.writeCell(Utils.getPappsoGroupingNumber(group, sg, k));
			odsTable.writeCell(prot.get_description());
			odsTable.writeCell(prot.get_evalue());
			// odsTable.writeCell("Coverage");
			odsTable.writeCell(match.get_coverage());
			// odsTable.writeCell("MW");
			odsTable.writeCell(prot.get_mw());
			// odsTable.writeCell("Spectra");
			odsTable.writeCell(hashProt.getTotalHashSampleScan());

			// odsTable.writeCell("Specific");
			int numberOfProtInSg = sg.getHashSampleScanSetProtSet().size();
			if (group.getSubGroupSet().size() > 1) {
				odsTable.writeCell(group
						.getNumberOfSpecificSpectraInSubGroup(hashProt));
			} else {
				// there is no specific scan, because there is only one subgroup
				odsTable.writeCell("");
			}
			// odsTable.writeCell("Uniques");
			odsTable.writeCell(hashProt.getUniquePeptideHashSampleScan());

			// odsTable.writeCell("Specific uniques");
			// if (numberOfProtInSg == 1) {
			if (group.getSubGroupSet().size() > 1) {
				odsTable.writeCell(group
						.getNumberOfSpecificUniquePeptideInSubGroup(hashProt));
			} else {
				// there is no specific scan, because there is only one subgroup
				odsTable.writeCell("");
			}

			odsTable.writeCell(prot.getNumberOfTrypticPeptidesForPAI());
			// odsTable.writeCell("PAI");
			// odsTable.writeCell(new Double((float) hashProt
			// .getTotalHashSampleScan() / prot.get_PAI_count()));
			odsTable.writeCell(hashProt.getPAI());
			// odsTable.writeCell(match.getPAI());//this is the same
			/*
			 * ancien PAI odsTable.writeCell(new Double((float)
			 * top.get_spectra_numbers_to(sample) /
			 * top.get_protein_match().get_PAI_count()));
			 */
			odsTable.writeCell(hashProt.getEmPAI());

			// odsTable.writeCell("Redundancy");
			odsTable.writeCell(numberOfProtInSg);

			odsTable.writeCell(hashProt.getGrpMsSampleSet().size());

			logger.debug("writeOneProtein end");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeOneProtein "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeOneProtein " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}
}
